
;; GETDECC.ASM -- a function to read a decimal number (with optional
;;   minus sign) from the keyboard and return the binary value in ax
;;   Program returns carry flag clear if no error.
;;
;;   This is the solution to exercise 7.3-8
;;
;;  calling sequence:
;;  EXTRN   GetDec: NEAR
;;  call    GetDec ;    On return, number read is in ax
;;          if carry clear; error if carry set
;;
;;  Program text from "Assembly Language for the IBM PC Family" by
;;   William B. Jones, (c) Copyright 1997, 2001, Scott/Jones Inc.
;;
        .MODEL  SMALL

INCLUDE PCMAC.INC

.DATA
Sign    DB      ?
Counter DB      ?

.CODE

PUBLIC	GetDec
GetDec	PROC
	push    bx ;        Don't need to save ax, but bx, cx, ...
        push    cx ;        ...dx must be saved and restored
        push    dx
        mov     Counter, 0 ;    number of digits read
        mov     bx, 0 ;         accumulated NumberValue in bx := 0
        mov     cx, 10
        mov     Sign, '+' ;     Guess that sign will be '+'
        _GetCh  ;               Read character ==> al
        cmp     al, '-' ;       Is first character a minus sign?
        jne     AfterRead
        mov     Sign, '-' ;      yes
ReadLoop:
        _GetCh
AfterRead:
        cmp     al, '0' ;       Is character a digit?
        jl      Done ;           No
        cmp     al, '9'
        jg      Done ;           No
        sub     al, '0' ;        Yes, cvt to DigitValue and extend to a
        mov     ah, 0 ;           word (so we can add it to NumberValue)
        inc     Counter ;        A digit has occurred; count it.
        xchg    ax, bx ;        Save DigitValue
        ;                        and set up NumberValue for mul
        mul     cx ;            NumberValue * 10 ...
        jo      Error
        add     ax, bx ;         + DigitValue ...
        mov     bx, ax ;           ==> NumberValue
        jc      Error
        jmp     ReadLoop
Done:
        cmp     al, 13 ;        If last character read was a RETURN...
        jne     NoLF
        _PutCh  10 ;             ...echo a matching line feed
NoLF:
        cmp     Counter, 0
        je      Error ;         No digits occurred
        cmp bx, 8000h ;         tricky way to do only one compare
        ja      Error ;          gets all negative numbers except 8000h
        jne     OK
        cmp     Sign, '-'
        je      Positive
Error:
        stc ;                   Set carry bit to indicate error
        jmp     Return ;         and return to caller
OK:
        cmp     Sign, '-'
        jne     Positive
        neg     bx ;            Final result is in bx
Positive:
        mov     ax, bx ;        Returned value --> ax
        clc ;                   Clear carry bit
Return:
        pop     dx ;        restore registers
        pop     cx
        pop     bx
        ret
GetDec  ENDP
END
